export const BULLET = "\u23FA";

export const INDICATOR_SELECTED = "\u25B8 ";
export const INDICATOR_UNSELECTED = "  ";
export const CURSOR_CHAR = "\u2588";
export const CHECKBOX_CHECKED = "[\u2022] ";
export const CHECKBOX_UNCHECKED = "[ ] ";

export const MAX_MESSAGES = 200;
export const MAX_TOOL_MESSAGE_CHARS = 4000;
export const MAX_ASSISTANT_CHARS = 50000;

export const MAX_DIFF_LINES = 6;

export const MAX_TOOL_OUTPUT_LINES = 3;
export const MAX_TOOL_DESCRIPTION_CHARS = 120;
export const MAX_TOOL_RESULT_LINE_CHARS = 60;
export const MIN_RESEARCH_DURATION_SHOW = 5;

export const INDEX_STATUS_POLL_MS = 500;
export const INDEX_DONE_HIDE_MS = 1000;

export const Status = {
  IDLE: "idle",
  THINKING: "thinking",
  COMPRESSING: "compressing",
  TOOL: "tool",
  AWAITING_APPROVAL: "awaiting_approval",
} as const;

export type Status = (typeof Status)[keyof typeof Status];
